#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#define F_CPU 16000000
#include <util/delay.h>

#define LINES PORTB
#define COLS0_7 PORTC
#define COLS8_9 PORTA
#define BUTTONS_PORT PORTA
#define BUTTONS PINA

#define BUTTON_UP PA0
#define BUTTON_RIGHT PA1
#define BUTTON_MIDDLE PA2
#define BUTTON_LEFT PA3
#define BUTTON_DOWN PA4

#define LINES_DDR DDRB
#define COLS0_7_DDR DDRC
#define COLS8_9_DDR DDRA
#define BUTTONS_DDR DDRA

#define LED_LIGHT_TIME 0.1

#define BLINK_FREQ 40

int8_t display[10][10];

uint8_t blink_count;

void clear_display()
{
	int i, j;
	
	for (i = 0; i < 10; i++)
	{
		for (j = 0; j < 10; j++)
		{
			display[i][j] = 0;
		}
	}
}

void refresh_display()
{
	uint8_t i, j;
	
	for (i = 0; i < 10; i++)
	{
		//Select line
		LINES &= 0x00;
		LINES |= i;
		
		//Init columns
		COLS0_7 &= 0x00;
		COLS8_9 &= 0x1F;

		for (j = 0; j < 10; j++)
		{
			if(j >= 8)
			{
				switch (display[i][j])
				{
					case 1:
						COLS8_9 |= _BV(j-2);
					break;
					case 2:
						if (blink_count == BLINK_FREQ)
						{
							COLS8_9 |= _BV(j-2);
							display[i][j] = 3;
						}
					break;
					case 3:
						COLS8_9 |= _BV(j-2);
						if (blink_count == BLINK_FREQ)
						{
							display[i][j] = 2;
						}
					break;
				}
				
				//Leave LED on for AIR_TIME ms
				_delay_ms(LED_LIGHT_TIME);
			
				//Switch off LED
				COLS8_9 &= 0x1F;
				
				continue;
			}
			
			switch (display[i][j])
			{
				case 1:
					COLS0_7 |= _BV(j);
				case 2:
					COLS0_7 |= _BV(j);
					if (blink_count == BLINK_FREQ)
					{
						display[i][j] = 3;
					}
				break;
				case 3:
					COLS0_7 |= _BV(j);
					if (blink_count == BLINK_FREQ)
					{
						display[i][j] = 2;
					}
				break;
			}

			//Leave LED on for AIR_TIME ms
			_delay_ms(LED_LIGHT_TIME);
			
			//Switch off LED
			COLS0_7 &= 0x00;
		}
	}
					
}


void init_timer()
{
	sei();
	OCR1A = 16;
	TCCR1A |= _BV(WGM10);
	TCCR1B |= _BV(WGM13);
	TIMSK |= _BV(OCIE1A);
	blink_count = 0;
}

ISR(TIMER1_COMPA_vect)
{	
	blink_count++;
	
	if (blink_count > BLINK_FREQ)
	{
		blink_count = 0;
	}
	refresh_display();
}

/*
//TODO:
uint8_t validate_position(uint8_t x, uint8_t y, uint8_t dir)
{
	
}
*/

//TODO:
void draw_plane(uint8_t x, uint8_t y, uint8_t dir)
{
		
}

int main()
{	
	uint8_t cursor_x = 4, cursor_y = 4;
	uint8_t cursor_type = 2;
	
	LINES_DDR |= 0xFF;
	COLS0_7_DDR |= 0xFF;
	COLS8_9_DDR |= 0xFF;
	BUTTONS_DDR &= 0xC0;
	
	LINES &= 0x00;
	COLS0_7 &= 0x00;
	COLS8_9 &= 0x0F;
	BUTTONS_PORT |= 0x1F;
	
 	clear_display();

	init_timer();
	
	display[cursor_x][cursor_y] = 2;
	
	while(1)
	{
		if ((BUTTONS & 0xFF) != 0xFF)
		{
			display[cursor_x][cursor_y] = 0;
			
			if (!(BUTTONS & _BV(BUTTON_LEFT)))
			{
				cursor_y--;
			}
			if (!(BUTTONS & _BV(BUTTON_RIGHT)))
			{	
				cursor_y++;
			}
			if (!(BUTTONS & _BV(BUTTON_DOWN)))
			{
				cursor_x++;
			}
			if (!(BUTTONS & _BV(BUTTON_UP)))
			{
				cursor_x--;
			}
			
			if (!(PINA & _BV(BUTTON_MIDDLE)))
			{
				//cursor_type = cursor_type == 1? 2 : 1;
			}
			
			if (cursor_x < 0)
				cursor_x = 0;
			if (cursor_x > 9)
				cursor_x = 9;
			if (cursor_y < 0)
				cursor_y = 0;
			if (cursor_y > 9)
				cursor_y = 9;
				
			display[cursor_x][cursor_y] = cursor_type;
		}
		
		//TODO: listen or ignore buttons
		//TODO: serial data write/read
	}

	return 0;
}