#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/eeprom.h>
#include <string.h>
#include "ports_used_cristi.h"
 
//LED display refresh time and blinking led timing
#define REFRESH_TIME 5000	//uS
#define BLINK_FREQ 10

//Pixel types
#define PIXEL_OFF 0
#define PIXEL_ON 1
#define PIXEL_BLINK 2

//Directions for draw_plane function
#define PLANE_UP 1
#define PLANE_DOWN 2
#define PLANE_RIGHT 3
#define PLANE_LEFT 4

//Button indexes for button_pushed vector
#define LEFT_PUSHED 0
#define RIGHT_PUSHED 1
#define UP_PUSHED 2
#define DOWN_PUSHED 3
#define MIDDLE_PUSHED 4
#define FUNCTION 5
#define TIMEOUT 150

// --- Program states --- //
#define SETUP_CHOICE 0
#define SETUP_SAVE 1
#define CONNECT 2
#define IN_GAME 3
#define GAME_END 4
#define IDLE 7

#define VALIDATE_BOUNDARY 1
#define VALIDATE_COMPLETE 2

uint8_t display_load;
uint8_t game_state;
uint16_t blink_count;
uint8_t blink_state;

uint8_t display[10][10];
uint8_t dump[10][10];
uint8_t hits[10][10];



typedef struct plane
{
	uint8_t x, y, status;
	int8_t dir;
}plane_t;

plane_t planes[3];
uint8_t planes_no;

typedef struct dot
{
	uint8_t x, y;
}dot_t;

dot_t plane_dots[10];

void clear_display()
{
	uint8_t i, j;
	
	display_load = 0;
	
	for (i = 0; i < 10; i++)
	{
		for (j = 0; j < 10; j++)
		{
			display[i][j] = 0;
		}
	}
}

void my_delay()
{
//	float time = REFRESH_TIME / display_load;
//	uint16_t us = (int) floor(time);
	uint16_t us = REFRESH_TIME / display_load;
	uint16_t i;
	
	for (i = 0; i < us; i++)
	{
		_delay_loop_1(F_CPU / 1000000);
	}
}

void init_random()
{
	uint32_t ee = eeprom_read_dword(0);
	srandom(ee);
	uint32_t nr = random();
	eeprom_write_dword(0, nr);
}

void refresh_display()
{
	uint8_t  i, j;
	
	for (i = 0; i < 10; i++)
	{
		//Select line
		LINES &= 0x00;
		LINES = i;
		
		for (j = 0; j < 10; j++)
		{
			//Init columns
			COLS0_7 &= 0x00;
			COLS8_9 &= 0x0F;
	
			switch (display[i][j])
			{
				case PIXEL_ON:
					COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
					COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					
					my_delay(); //Leave LED on for LED_LIGHT_TIME ms
				break;
				case PIXEL_BLINK:
					if (blink_state)
					{
						COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
						COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					}
					my_delay();
				break;
// 					if (blink_count == BLINK_FREQ)
// 					{
// 						display[i][j] = PIXEL_BLINK + 1;
// 					}
// 				break; 
// 				case PIXEL_BLINK + 1:
// 					my_delay();
// 					
// 					if (blink_count == BLINK_FREQ)
// 					{
// 						display[i][j] = PIXEL_BLINK;
// 					}
// 				break;
			}
		}
	}
}

uint8_t validate_dot_position(uint8_t x, uint8_t y)
{
	if((x < 0) || (x > 9) || (y < 0) || (y > 9))
	{
		return 1;
	}
	
	return 0;
}

void draw_dot(uint8_t x, uint8_t y, uint8_t  led_value)
{

	display[x][y] = led_value ;
	
	if (led_value != PIXEL_OFF)
	{
		display_load++;
	}
	else
	{
		display_load--;
	}
	
}


void get_plane_dots(dot_t *vect, uint8_t x, uint8_t y, int8_t dir)
{
	switch(dir)
	{
		case PLANE_UP:
			vect[0].x = x-1;			
			vect[0].y = y;
			
			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x+1;
			vect[6].y = y;

			vect[7].x = x+2;
			vect[7].y = y-1;
			vect[8].x = x+2;
			vect[8].y = y;
			vect[9].x = x+2;
			vect[9].y = y+1;
		break;
		case PLANE_DOWN:
			vect[0].x = x+1;			
			vect[0].y = y;

			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x-1;
			vect[6].y = y;

			vect[7].x = x-2;
			vect[7].y = y-1;
			vect[8].x = x-2;
			vect[8].y = y;
			vect[9].x = x-2;
			vect[9].y = y+1;
		break;
		case PLANE_RIGHT:
			vect[0].x = x;			
			vect[0].y = y+1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y-1;

			vect[7].x = x-1;
			vect[7].y = y-2;
			vect[8].x = x;
			vect[8].y = y-2;
			vect[9].x = x+1;
			vect[9].y = y-2;
		break;
		case PLANE_LEFT:
			vect[0].x = x;			
			vect[0].y = y-1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y+1;

			vect[7].x = x-1;
			vect[7].y = y+2;
			vect[8].x = x;
			vect[8].y = y+2;
			vect[9].x = x+1;
			vect[9].y = y+2;
		break;					
	}	
}

uint8_t validate_plane_position(uint8_t x, uint8_t y, int8_t dir, uint8_t validate)
{
	get_plane_dots(plane_dots, x, y, dir);
	dot_t tmp_dots[10];
	
	uint8_t i, c, j;
	
	for(i = 0; i < 10; i++)
	{
		if(validate_dot_position(plane_dots[i].x, plane_dots[i].y) == 1)
		{
			return 1;
		}
	}
	
	if (validate == VALIDATE_COMPLETE)
	{
		
		for (c = 0; c < planes_no; c++)
		{
			get_plane_dots(tmp_dots, planes[c].x, planes[c].y, planes[c].dir);
			for (i = 0; i < 10;  i++)
			{
				for (j = 0; j < 10; j++)
				{
					if ((plane_dots[i].x == tmp_dots[j].x) && (plane_dots[i].y == tmp_dots[j].y))
					{
						return 1;
					}
				}
			}
		}
	}
	
	return 0;
}


void draw_plane(uint8_t x, uint8_t y, int8_t dir, uint8_t led_value)
{	
	uint8_t i;

	get_plane_dots(plane_dots, x, y, dir);
	
	for(i = 0; i<10; i++)
	{
		draw_dot(plane_dots[i].x, plane_dots[i].y, led_value);
	}
}


dot_t  random_plane_position(int8_t *dir)
{
	dot_t pos;
	uint8_t misses = 0;
	
	pos.x = random() % 10;
	pos.y = random() % 10;
	*dir = PLANE_UP;
	
	while(validate_plane_position(pos.x, pos.y, *dir, VALIDATE_COMPLETE) == 1)
	{
		misses++;
		
		if (misses == 100)
		{
			*dir += 1;
			misses = 0;
			if (*dir > PLANE_LEFT)
			{
				break;
			}
		}
		
		pos.x = random() % 10;
		pos.y = random() % 10;
	}
	
	return pos;
}

dot_t random_cursor_position()
{
	dot_t pos;
	
	pos.x = random() %10;
	pos.y = random() % 10;
	
	while(validate_dot_position(pos.x, pos.y) == 1)
	{
		pos.x = random() %10;
		pos.y = random() % 10;
	}
	
	return pos;
}

void init_game(uint8_t *start_x, uint8_t *start_y, int8_t *dir, uint8_t *dot_type, uint8_t *saved_dot_type, uint8_t *planes_no)
{
	LINES_DDR |= 0xFF;
	COLS0_7_DDR |= 0xFF;
	COLS8_9_DDR |= 0xC0;
	BUTTONS_DDR |= 0xC0;
	
	LINES &= 0x00;
	COLS0_7 &= 0x00;
	COLS8_9 &= 0x00;
	BUTTONS_PORT |= 0x1F;
	 
 	clear_display();
	init_random();
	
	game_state = SETUP_CHOICE;
	
	dot_t tmp;
	
	*dir = PLANE_UP;
	tmp = random_plane_position(dir);
	*start_x = tmp.x;
	*start_y = tmp.y;
	
	blink_count = 0;
	blink_state = 0;
	*dot_type = PIXEL_BLINK;
	*saved_dot_type = *dot_type;
	
	*planes_no = 1;
	draw_plane(*start_x, *start_y, *dir, *dot_type);
}

int main()
{
	uint8_t cursor_x, cursor_y, dot_type, saved_dot_type, dot_under_type = PIXEL_OFF ;
	uint8_t opponent_cursor;
	int8_t plane_dir;
	int8_t button_pushed[6] = {0, 0, 0, 0, 0, 0};
	uint32_t timeout = 0, timeout_on = 0;
	uint8_t i;		//temp

	init_game(&cursor_x, &cursor_y, &plane_dir, &dot_type, &saved_dot_type, &planes_no);
	
	while(1)
	{
		
			switch(game_state)
			{
				case SETUP_CHOICE:
					
					dot_type = PIXEL_ON;
					//saved_dot_type = display[cursor_x][cursor_y];
					
					for (i = 0; i < planes_no; i++)
					{
						draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_OFF);
						draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_ON);
					}
					
					dot_type = PIXEL_OFF;										
					draw_plane(cursor_x, cursor_y, plane_dir, dot_type);
					
				break;
				case SETUP_SAVE:
					if(planes_no < 3)
					{
						planes[planes_no].x = cursor_x;
						planes[planes_no].y = cursor_y;
						planes[planes_no].status = 0;
						planes[planes_no].dir = plane_dir;
						planes_no++;
						
						draw_plane(cursor_x, cursor_y, plane_dir, dot_type);
						
						dot_t tmp = random_plane_position(&plane_dir);
						cursor_x = tmp.x;
						cursor_y = tmp.y;	
						display[cursor_x][cursor_y] = PIXEL_BLINK;	//for new SETUP_CHOICE state saved_dot_type initialization
						game_state = SETUP_CHOICE;
					}
					else
					{
						//Setup finished, connecting to oponent's gizmo
						game_state = CONNECT;
					}
				break;
				case CONNECT:
					//Nuthin yet
					
				//Testing "hits" display:
					//Copy planes setup display
					memcpy(dump, display, 100 * sizeof(uint8_t));
					clear_display();
					
					//Generate cursor and random hits just to see if old Blinky can take it :)
					dot_t tmp = random_cursor_position();
					cursor_x = tmp.x;
					cursor_y = tmp.y;
					draw_dot(cursor_x, cursor_y, PIXEL_BLINK);

					for (i = 0; i < 15; i++)
					{
						tmp = random_cursor_position();
						if (tmp.x != cursor_x || tmp.y != cursor_y)
							draw_dot(tmp.x, tmp.y, PIXEL_ON);
					}
					game_state = IN_GAME;
					;
				break;
				case IN_GAME:
					; //Nuthin yet
				break;
			} 
			
			//Button readings
			switch (game_state)
			{
				
				case SETUP_CHOICE:
				//Button release events:
					//draw_plane(cursor_x, cursor_y, plane_dir, saved_dot_type);
					
					//Left button
					if ((BUTTONS & _BV(BUTTON_LEFT)) && button_pushed[LEFT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_LEFT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_LEFT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  - 1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y--;
							}					
						}
						button_pushed[LEFT_PUSHED] = 0;				
					}
			
					//Right button		
					if ((BUTTONS & _BV(BUTTON_RIGHT)) && button_pushed[RIGHT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_RIGHT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_RIGHT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  +1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y++;
							}
						}
						button_pushed[RIGHT_PUSHED] = 0;				
					}	
			
					//Up button
					if((BUTTONS & _BV(BUTTON_UP)) && button_pushed[UP_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_UP, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_UP;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x - 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x--;
							}					
						}
						button_pushed[UP_PUSHED] = 0;			
					}	
			
					//Down button
					if ((BUTTONS & _BV(BUTTON_DOWN))  && button_pushed[DOWN_PUSHED] )
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_DOWN, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_DOWN;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x + 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x++;
							}					
						}
						button_pushed[DOWN_PUSHED] = 0;
					}	
			
					//Middle button
					if ((BUTTONS & _BV(BUTTON_MIDDLE)) && button_pushed[MIDDLE_PUSHED])
					{	
						button_pushed[FUNCTION]++;
				
						if (button_pushed[FUNCTION] == 2)
						{
							if (validate_plane_position(cursor_x, cursor_y, plane_dir, VALIDATE_COMPLETE) != 1)
							{
								game_state = SETUP_SAVE;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							timeout = 0;
							timeout_on = 1;
						}
						button_pushed[MIDDLE_PUSHED] = 0;
					}	

			//Button press events:
			
					switch(~BUTTONS & 0x1F)
					{
						case _BV(BUTTON_RIGHT):
							if (!button_pushed[RIGHT_PUSHED])
								button_pushed[RIGHT_PUSHED] = 1;
						break;
						case _BV(BUTTON_LEFT):
							if (!button_pushed[LEFT_PUSHED])
								button_pushed[LEFT_PUSHED] = 1;
						break;
						case _BV(BUTTON_UP):
							if (!button_pushed[UP_PUSHED])
								button_pushed[UP_PUSHED] = 1;
						break;
						case _BV(BUTTON_DOWN):
							if (!button_pushed[DOWN_PUSHED])
								button_pushed[DOWN_PUSHED] = 1;				
						break;
						case _BV(BUTTON_MIDDLE):
							if (!button_pushed[MIDDLE_PUSHED])
								button_pushed[MIDDLE_PUSHED] = 1;				
						break;
					}
				break;
			}
			switch(game_state)
			{
				case SETUP_CHOICE:
					//dot_under_type = display[cursor_x][cursor_y];
					draw_plane(cursor_x, cursor_y, plane_dir, PIXEL_BLINK);
				break;
				case SETUP_SAVE:
					dot_type = PIXEL_ON;
				break;
			}
			
		
		//Keyboard timeout
		if (timeout_on)
		{
			timeout++;	
			if (timeout == TIMEOUT)
			{
				button_pushed[FUNCTION] = 0;
				//PORTD ^= _BV(PD7);
			}
		}
		
		// Refresh display //
		blink_count++;
		refresh_display();
		blink_count %= BLINK_FREQ;
		if (blink_count == 0)
		{
			blink_state = (blink_state + 1) % 2;
			PORTD ^= _BV(PD7);
		}
		
		//-----------------------//
		
		}
	

	return 0;
}