/* Backup created at:
Fri Jun  4 03:15:20 EEST 2010*/

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/eeprom.h>
#include <string.h>
#include "ports_used_cristi.h"
 
//LED display refresh time and blinking led timing
#define REFRESH_TIME 5000	//uS
#define BLINK_FREQ 10

//Pixel types
#define PIXEL_OFF 0
#define PIXEL_ON 1
#define PIXEL_BLINK 2

//Directions for draw_plane function
#define PLANE_UP 1
#define PLANE_DOWN 2
#define PLANE_RIGHT 3
#define PLANE_LEFT 4

//Button indexes for button_pushed vector
#define LEFT_PUSHED 0
#define RIGHT_PUSHED 1
#define UP_PUSHED 2
#define DOWN_PUSHED 3
#define MIDDLE_PUSHED 4
#define FUNCTION 5
#define TIMEOUT 150

// --- Program states --- //
#define SETUP_CHOICE 0
#define SETUP_SAVE 1
#define CONNECT 2
#define IN_GAME 3
#define CHANGE_TURNS 4
#define GAME_END 6
#define IDLE 7

#define VALIDATE_BOUNDARY 1
#define VALIDATE_COMPLETE 2

#define PLANES_NUM 3

#define MY_TURN 1
#define OPP_TURN 2
#define MAX_HITS 70

//Transmission codes
#define SHOT_MISS	0xC0
#define SHOT_HIT	0xC1
#define SHOT_KILL	0xC2
#define SHOT_DECIDED	0xC3
#define ACK	0xCA

uint8_t display_load;
uint8_t game_state;
uint16_t blink_count;
uint8_t blink_state;

uint8_t map[10][10];
uint8_t hits[10][10];
uint8_t (*display)[10];

typedef struct plane_data
{
	uint8_t x, y;
	uint8_t status;
	uint8_t dir;
}plane_t;

typedef struct dot
{
	uint8_t x, y;
}dot_t;

typedef struct 
{
	uint8_t x, y;
	uint8_t type;
}hit_t;

plane_t planes[PLANES_NUM];
dot_t hits[MAX_HITS];
uint8_t plane_index;
uint8_t hit_index;


dot_t plane_dots[10];

void clear_display()
{
	uint8_t i, j;
	
	display_load = 0;
	
	for (i = 0; i < 10; i++)
	{
		for (j = 0; j < 10; j++)
		{
			display[i][j] = 0;
		}
	}
}

void my_delay()
{
//	float time = REFRESH_TIME / display_load;
//	uint16_t us = (int) floor(time);
	uint16_t us = REFRESH_TIME / display_load;
	uint16_t i;
	
	for (i = 0; i < us; i++)
	{
		_delay_loop_1(F_CPU / 1000000);
	}
}

void init_random()
{
	uint32_t ee = eeprom_read_dword(0);
	srandom(ee);
	uint32_t nr = random();
	eeprom_write_dword(0, nr);
}

void refresh_display()
{
	uint8_t  i, j;
	
	for (i = 0; i < 10; i++)
	{
		//Select line
		LINES &= 0x00;
		LINES = i;
		
		for (j = 0; j < 10; j++)
		{
			//Init columns
			COLS0_7 &= 0x00;
			COLS8_9 &= 0x0F;
	
			switch (display[i][j])
			{
				case PIXEL_ON:
					COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
					COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					
					my_delay(); //Leave LED on for LED_LIGHT_TIME ms
				break;
				case PIXEL_BLINK:
					if (blink_state)
					{
						COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
						COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					}
					my_delay();
				break;
			}
		}
	}
}

uint8_t validate_dot_position(uint8_t x, uint8_t y)
{
	if((x < 0) || (x > 9) || (y < 0) || (y > 9))
	{
		return 1;
	}
	
	return 0;
}

void draw_dot(uint8_t x, uint8_t y, uint8_t  led_value)
{

	display[x][y] = led_value ;
	
	if (led_value != PIXEL_OFF)
	{
		display_load++;
	}
	else
	{
		display_load--;
	}
	
}


void get_plane_dots(dot_t *vect, uint8_t x, uint8_t y, uint8_t dir)
{
	switch(dir)
	{
		case PLANE_UP:
			vect[0].x = x-1;			
			vect[0].y = y;
			
			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x+1;
			vect[6].y = y;

			vect[7].x = x+2;
			vect[7].y = y-1;
			vect[8].x = x+2;
			vect[8].y = y;
			vect[9].x = x+2;
			vect[9].y = y+1;
		break;
		case PLANE_DOWN:
			vect[0].x = x+1;			
			vect[0].y = y;

			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x-1;
			vect[6].y = y;

			vect[7].x = x-2;
			vect[7].y = y-1;
			vect[8].x = x-2;
			vect[8].y = y;
			vect[9].x = x-2;
			vect[9].y = y+1;
		break;
		case PLANE_RIGHT:
			vect[0].x = x;			
			vect[0].y = y+1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y-1;

			vect[7].x = x-1;
			vect[7].y = y-2;
			vect[8].x = x;
			vect[8].y = y-2;
			vect[9].x = x+1;
			vect[9].y = y-2;
		break;
		case PLANE_LEFT:
			vect[0].x = x;			
			vect[0].y = y-1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y+1;

			vect[7].x = x-1;
			vect[7].y = y+2;
			vect[8].x = x;
			vect[8].y = y+2;
			vect[9].x = x+1;
			vect[9].y = y+2;
		break;					
	}	
}

uint8_t validate_plane_position(uint8_t x, uint8_t y, uint8_t dir, uint8_t validate)
{
	get_plane_dots(plane_dots, x, y, dir);
	dot_t tmp_dots[10];
	
	uint8_t i, c, j;
	
	for(i = 0; i < 10; i++)
	{
		if(validate_dot_position(plane_dots[i].x, plane_dots[i].y) == 1)
		{
			return 1;
		}
	}
	
	if (validate == VALIDATE_COMPLETE)
	{
		
		for (c = 0; c < plane_index; c++)
		{
			get_plane_dots(tmp_dots, planes[c].x, planes[c].y, planes[c].dir);
			for (i = 0; i < 10;  i++)
			{
				for (j = 0; j < 10; j++)
				{
					if ((plane_dots[i].x == tmp_dots[j].x) && (plane_dots[i].y == tmp_dots[j].y))
					{
						return 1;
					}
				}
			}
		}
	}
	
	return 0;
}


void draw_plane(uint8_t x, uint8_t y, uint8_t dir, uint8_t led_value)
{	
	uint8_t i;

	get_plane_dots(plane_dots, x, y, dir);
	
	for(i = 0; i<10; i++)
	{
		draw_dot(plane_dots[i].x, plane_dots[i].y, led_value);
	}
}

dot_t random_cursor_position()
{
	dot_t pos;
	
	pos.x = random() % 10;
	pos.y = random() % 10;
	
	while(validate_dot_position(pos.x, pos.y) == 1)
	{
		pos.x = random() % 10;
		pos.y = random() % 10;
	}
	
	return pos;
}

dot_t  random_plane_position(uint8_t *dir)
{
	uint8_t misses = 0;	
	dot_t pos = random_cursor_position();
	
	while(validate_plane_position(pos.x, pos.y, *dir, VALIDATE_COMPLETE) == 1)
	{
		misses++;
		
		if (misses == 100)
		{
			*dir += 1;
			misses = 0;
			
			if (*dir > PLANE_LEFT)
			{
				break;
			}
		}
		
		pos = random_cursor_position();
	}
	
	return pos;
}

void init_game(uint8_t *start_x, uint8_t *start_y, uint8_t *dir, uint8_t *dot_type, uint8_t *saved_dot_type)
{
	LINES_DDR |= 0xFF;
	COLS0_7_DDR |= 0xFF;
	COLS8_9_DDR |= 0xC0;
	BUTTONS_DDR |= 0xC0;
	
	LINES &= 0x00;
	COLS0_7 &= 0x00;
	COLS8_9 &= 0x00;
	BUTTONS_PORT |= 0x1F;
	 
 	display = map;
 	clear_display();
	init_random();
	
	game_state = SETUP_CHOICE;
	*dir = PLANE_UP;
	
	dot_t tmp = random_plane_position(dir);
	*start_x = tmp.x;
	*start_y = tmp.y;
	
	blink_count = 0;
	blink_state = 0;
	
	*dot_type = PIXEL_BLINK;
	*saved_dot_type = *dot_type;
	
	plane_index = 0;
	hit_index = 0;
	draw_plane(*start_x, *start_y, *dir, *dot_type);
}

int main()
{
	uint8_t cursor_x, cursor_y/*, opponent_cursor_x, opponent_cursor_y*/;

	uint8_t dot_type, saved_dot_type;
	
	uint8_t plane_dir;

	uint8_t i;	
	int8_t button_pushed[6] = {0, 0, 0, 0, 0, 0};

	uint32_t timeout = 0, timeout_on = 0;
	
	uint8_t not_connected = 1;	
	uint8_t turn = 0;	
	
	init_game(&cursor_x, &cursor_y, &plane_dir, &dot_type, &saved_dot_type);
	
	while (not_connected)
	{
		//Animation :D
		
		
		// if timeout_on, throw dice (random() % 10) and send value and set timeout_on = 0
		
		//Nu wait ci se face o verificare a flag-ului daca s-a primit un mesaj.
			//Wait for ACK. Daca primeste dice in loc de ACK atunci ia in considerare ce a primit de la oponent, trimite ACK si trimite dice si asteapta ACK, stabileste cine incepe primu
			//Daca primeste ACK, asteapta sa primeasca dice, confirma si stablieste cine incepe primu
		
		//Dupa ce a stabilit cine incepe primu, not_connected = 0, continue;
		//refresh_display() pentru animatie
	
	}
	
	while(1)
	{
			switch(game_state)
			{
				case SETUP_CHOICE:
					for (i = 0; i < plane_index; i++)
					{
						draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_OFF);
						draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_ON);
					}
					
					dot_type = PIXEL_OFF;										
					draw_plane(cursor_x, cursor_y, plane_dir, dot_type);
					
				break;
				case SETUP_SAVE:
					if(plane_index < PLANES_NUM)
					{
						planes[plane_index].x = cursor_x;
						planes[plane_index].y = cursor_y;
						planes[plane_index].status = 0;
						planes[plane_index].dir = plane_dir;
						plane_index++;
						
						draw_plane(cursor_x, cursor_y, plane_dir, PIXEL_ON);
					}						
				
					if (plane_index != PLANES_NUM)
					{
						dot_t tmp = random_plane_position(&plane_dir);
						cursor_x = tmp.x;
						cursor_y = tmp.y;	
						
						display[cursor_x][cursor_y] = PIXEL_BLINK;	//for new SETUP_CHOICE state saved_dot_type initialization

						game_state = SETUP_CHOICE;
					}
					else
					{
						game_state = CONNECT;
					}
				break;
				case IN_GAME:
					; //Nuthin yet
					
					switch (turn)
					{
						case MY_TURN:
							//display points to hits matrix
							for (i = 0; i < hit_index; i++)
							{
								draw_dot(hits[i].x, hits[i].y, PIXEL_OFF);
								draw_dot(hits[i].x, hits[i].y, PIXEL_ON);
							}
					
							dot_type = PIXEL_OFF;										
							draw_dot(cursor_x, cursor_y, dot_type);

						break;
						
						case OPP_TURN:
							//display points to map matrix
							for (i = 0; i < PLANES_NUM; i++)
							{
								draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_OFF);
								draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_ON);
							}
					
							dot_type = PIXEL_OFF;										
							draw_dot(cursor_x, cursor_y, dot_type);							
						break;
					}
				break;
				
				case CHANGE_TURNS:
					if (turn == MY_TURN)
					{
						turn = OPP_TURN;
						display = map;
					}
					if (turn == OPP_TURN)
					{
						turn = MY_TURN;
						display = hits;
					}
					
					state = IN_GAME;
				break;
			} 
			
			//Button readings
			switch (game_state)
			{
				case SETUP_CHOICE:
				//Button release events:
					//Left button
					if ((BUTTONS & _BV(BUTTON_LEFT)) && button_pushed[LEFT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_LEFT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_LEFT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  - 1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y--;
							}					
						}
						button_pushed[LEFT_PUSHED] = 0;				
					}
			
					//Right button		
					if ((BUTTONS & _BV(BUTTON_RIGHT)) && button_pushed[RIGHT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_RIGHT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_RIGHT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  +1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y++;
							}
						}
						button_pushed[RIGHT_PUSHED] = 0;				
					}	
			
					//Up button
					if((BUTTONS & _BV(BUTTON_UP)) && button_pushed[UP_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_UP, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_UP;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x - 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x--;
							}					
						}
						button_pushed[UP_PUSHED] = 0;			
					}	
			
					//Down button
					if ((BUTTONS & _BV(BUTTON_DOWN))  && button_pushed[DOWN_PUSHED] )
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_DOWN, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_DOWN;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x + 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x++;
							}					
						}
						button_pushed[DOWN_PUSHED] = 0;
					}	
			
					//Middle button
					if ((BUTTONS & _BV(BUTTON_MIDDLE)) && button_pushed[MIDDLE_PUSHED])
					{	
						button_pushed[FUNCTION]++;
				
						if (button_pushed[FUNCTION] == 2)
						{
							if (validate_plane_position(cursor_x, cursor_y, plane_dir, VALIDATE_COMPLETE) != 1)
							{
								game_state = SETUP_SAVE;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							timeout = 0;
							timeout_on = 1;
						}
						button_pushed[MIDDLE_PUSHED] = 0;
					}	
				break;
				
				case IN_GAME:
					switch (turn)
					{
						case MY_TURN:
							//move cursor and shoot

							//Left button
							if ((BUTTONS & _BV(BUTTON_LEFT)) && button_pushed[LEFT_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x, cursor_y  - 1)  != 1)
								{
									cursor_y--;
								}					
								button_pushed[LEFT_PUSHED] = 0;				
							}
			
							//Right button		
							if ((BUTTONS & _BV(BUTTON_RIGHT)) && button_pushed[RIGHT_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x, cursor_y  +1) != 1)
								{
									cursor_y++;
								}

								button_pushed[RIGHT_PUSHED] = 0;				
							}	
			
							//Up button
							if((BUTTONS & _BV(BUTTON_UP)) && button_pushed[UP_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x - 1, cursor_y) != 1)
								{
									cursor_x--;
								}					
								button_pushed[UP_PUSHED] = 0;			
							}	
			
							//Down button
							if ((BUTTONS & _BV(BUTTON_DOWN))  && button_pushed[DOWN_PUSHED] )
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x + 1, cursor_y) != 1)
								{
									cursor_x++;
								}					
								button_pushed[DOWN_PUSHED] = 0;
							}	
			
							//Middle button
							if ((BUTTONS & _BV(BUTTON_MIDDLE)) && button_pushed[MIDDLE_PUSHED])
							{	
								button_pushed[FUNCTION]++;
				
								if (button_pushed[FUNCTION] == 2)
								{
									if (validate_dot_position(cursor_x, cursor_y) != 1)
									{
										//Hit target
										game_state = CHANGE_TURNS;
									}
									button_pushed[FUNCTION] = 0;
									timeout_on = 0;
								}
								else
								{
									timeout = 0;
									timeout_on = 1;
								}
								button_pushed[MIDDLE_PUSHED] = 0;
							}	
						break;
						
						case OPP_TURN:
							//nothing, just sit and watch
							;
						break;
					}
				break;
			}
		//Button press events:
			switch(~BUTTONS & 0x1F)
			{
				case _BV(BUTTON_RIGHT):
					if (!button_pushed[RIGHT_PUSHED])
					{
						button_pushed[RIGHT_PUSHED] = 1;
					}
				break;
				case _BV(BUTTON_LEFT):
					if (!button_pushed[LEFT_PUSHED])
					{
						button_pushed[LEFT_PUSHED] = 1;
					}
				break;
				case _BV(BUTTON_UP):
					if (!button_pushed[UP_PUSHED])
					{
						button_pushed[UP_PUSHED] = 1;
					}
				break;
				case _BV(BUTTON_DOWN):
					if (!button_pushed[DOWN_PUSHED])
					{
						button_pushed[DOWN_PUSHED] = 1;				
					}
				break;
				case _BV(BUTTON_MIDDLE):
					if (!button_pushed[MIDDLE_PUSHED])
					{
						button_pushed[MIDDLE_PUSHED] = 1;
					}				
				break;
			}
			
			
			switch(game_state)
			{
				case SETUP_CHOICE:
						draw_plane(cursor_x, cursor_y, plane_dir, PIXEL_BLINK);
				break;
				
				case IN_GAME:
					switch (turn)
					{
						case MY_TURN:
							//Send cursor position
						break;
						
						case OPP_TURN:
							//Receive cursor position
							//Update cursor and draw cursor
							//if SHOT_DECIDED, send feedback: SHOT_MISS, SHOT_HIT, state = CHANGE_TURNS
						break;
					}
				break;
				case CHANGE_TURNS:
					switch (turn)
					{
						case MY_TURN:
							//Send cursor position
							//Send SHOT_DECIDED
							//Receive feedback
						break;
					}
				break;
			}
			
		
		//Keyboard timeout
		if (timeout_on)
		{
			timeout++;	
			if (timeout == TIMEOUT)
			{
				button_pushed[FUNCTION] = 0;
				//PORTD ^= _BV(PD7);
			}
		}
		
		// Refresh display //
		blink_count++;
		refresh_display();
		blink_count %= BLINK_FREQ;
		if (blink_count == 0)
		{
			blink_state = (blink_state + 1) % 2;
			PORTD ^= _BV(PD7);
		}
		
		//-----------------------//
		
		}
	

	return 0;
}