#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <avr/eeprom.h>
#include <string.h>
#include "ports_used_cristi.h"
//~ #include "ports_used_dan.h" 
 

//LED display refresh time and blinking led timing
#define REFRESH_TIME 5000	//uS
#define BLINK_FREQ 10

//Pixel types
#define PIXEL_OFF 0
#define PIXEL_ON 1
#define PIXEL_BLINK 2

//Directions for draw_plane function
#define PLANE_UP 1
#define PLANE_DOWN 2
#define PLANE_RIGHT 3
#define PLANE_LEFT 4

//Button indexes for button_pushed vector
#define LEFT_PUSHED 0
#define RIGHT_PUSHED 1
#define UP_PUSHED 2
#define DOWN_PUSHED 3
#define MIDDLE_PUSHED 4
#define FUNCTION 5
#define TIMEOUT 150

// --- Program states --- //
#define SETUP_CHOICE 0
#define SETUP_SAVE 1
#define READY 2
#define IN_GAME 3
#define CHANGE_TURNS 4
#define SHOOT	5
#define GAME_END 6

#define IDLE 9

#define VALIDATE_BOUNDARY 1
#define VALIDATE_COMPLETE 2

#define PLANES_NUM 3

#define MY_TURN 1
#define OPP_TURN 2
#define MAX_HITS 70

//TODO: Debug
uint8_t turn = OPP_TURN;	
//~ uint8_t turn = MY_TURN;	

typedef struct
{
	uint8_t load;
	uint8_t **pixels;
} display_t;

//Transmission codes
#define SHOT_MISS	0xC0
#define SHOT_HIT	0xC1
#define SHOT_KILL	0xC2
#define SHOT_DECIDED	0xC3
#define SHOT_DELAYED	0xC4
#define ACK	0xCA

//~ uint8_t display_load;
//~ uint8_t display_load_save;
uint8_t game_state;
uint8_t skip_states;
uint16_t blink_count;
uint8_t blink_state;


// uint8_t map[10][10];
// uint8_t radar[10][10];
// uint8_t (*display)[10];

//~ uint8_t **map, **radar, **display;
display_t *map, *radar, *current_display;

typedef struct dot
{
	uint8_t x, y;
}dot_t;

typedef struct plane_data
{
	uint8_t x, y;
	uint8_t status;
	uint8_t dir;
	dot_t head;
}plane_t;

typedef struct 
{
	uint8_t x, y;
	uint8_t type;
}hit_t;

plane_t planes[PLANES_NUM];
hit_t hits[MAX_HITS];
uint8_t plane_index;
uint8_t hit_index;
uint8_t successful_hits;

dot_t plane_dots[10];

void clear_display()
{
	uint8_t i, j;
	
	current_display->load = 0;
	
	for (i = 0; i < 10; i++)
	{
		for (j = 0; j < 10; j++)
		{
			current_display->pixels[i][j] = 0;
		}
	}
}

void my_delay()
{
	uint16_t us = REFRESH_TIME / (current_display->load != 0 ? current_display->load : 1);
	uint16_t i;
	
	for (i = 0; i < us; i++)
	{
		_delay_loop_1(F_CPU / 1000000);
	}
}

void init_random()
{
	uint32_t ee = eeprom_read_dword(0);
	srandom(ee);
	uint32_t nr = random();
	eeprom_write_dword(0, nr);
}

void refresh_display()
{
	uint8_t  i, j;
	
	for (i = 0; i < 10; i++)
	{
		//Select line
		LINES &= 0x00;
		LINES = i;
		
		for (j = 0; j < 10; j++)
		{
			//Init columns
			COLS0_7 &= 0x00;
			COLS8_9 &= 0x0F;
	
			switch (current_display->pixels[i][j])
			{
				case PIXEL_ON:
					COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
					COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					
					my_delay(); //Leave LED on for LED_LIGHT_TIME ms
				break;
				case PIXEL_BLINK:
					if (blink_state)
					{
						COLS0_7 |= j <= 7 ? _BV(j) : 0x00;
						COLS8_9 |= j >= 8 ? _BV(j - 2) : 0x00;
					}
					my_delay();
				break;
			}
		}
	}
}

uint8_t validate_dot_position(uint8_t x, uint8_t y)
{
	if((x < 0) || (x > 9) || (y < 0) || (y > 9))
	{
		return 1;
	}
	
	return 0;
}

void draw_dot(uint8_t x, uint8_t y, uint8_t  led_value)
{

	current_display->pixels[x][y] = led_value ;
	
	if (led_value != PIXEL_OFF)
	{
		current_display->load++;
	}
	else
	{
		current_display->load--;
	}
	
}


void get_plane_dots(dot_t *vect, uint8_t x, uint8_t y, uint8_t dir)
{
	switch(dir)
	{
		case PLANE_UP:
			vect[0].x = x-1;			
			vect[0].y = y;
			
			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x+1;
			vect[6].y = y;

			vect[7].x = x+2;
			vect[7].y = y-1;
			vect[8].x = x+2;
			vect[8].y = y;
			vect[9].x = x+2;
			vect[9].y = y+1;
		break;
		case PLANE_DOWN:
			vect[0].x = x+1;			
			vect[0].y = y;

			vect[1].x = x;
			vect[1].y = y - 2;
			vect[2].x = x;
			vect[2].y = y - 1;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x;
			vect[4].y = y+1;
			vect[5].x = x;
			vect[5].y = y+2;
			
			vect[6].x = x-1;
			vect[6].y = y;

			vect[7].x = x-2;
			vect[7].y = y-1;
			vect[8].x = x-2;
			vect[8].y = y;
			vect[9].x = x-2;
			vect[9].y = y+1;
		break;
		case PLANE_RIGHT:
			vect[0].x = x;			
			vect[0].y = y+1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y-1;

			vect[7].x = x-1;
			vect[7].y = y-2;
			vect[8].x = x;
			vect[8].y = y-2;
			vect[9].x = x+1;
			vect[9].y = y-2;
		break;
		case PLANE_LEFT:
			vect[0].x = x;			
			vect[0].y = y-1;
			
			vect[1].x = x-2;
			vect[1].y = y;
			vect[2].x = x-1;
			vect[2].y = y;
			vect[3].x = x;
			vect[3].y = y ;
			vect[4].x = x+1;
			vect[4].y = y;
			vect[5].x = x+2;
			vect[5].y = y;
			
			vect[6].x = x;
			vect[6].y = y+1;

			vect[7].x = x-1;
			vect[7].y = y+2;
			vect[8].x = x;
			vect[8].y = y+2;
			vect[9].x = x+1;
			vect[9].y = y+2;
		break;					
	}	
}

uint8_t validate_plane_position(uint8_t x, uint8_t y, uint8_t dir, uint8_t validate)
{
	get_plane_dots(plane_dots, x, y, dir);
	dot_t tmp_dots[10];
	
	uint8_t i, c, j;
	
	for(i = 0; i < 10; i++)
	{
		if(validate_dot_position(plane_dots[i].x, plane_dots[i].y) == 1)
		{
			return 1;
		}
	}
	
	if (validate == VALIDATE_COMPLETE)
	{
		
		for (c = 0; c < plane_index; c++)
		{
			get_plane_dots(tmp_dots, planes[c].x, planes[c].y, planes[c].dir);
			for (i = 0; i < 10;  i++)
			{
				for (j = 0; j < 10; j++)
				{
					if ((plane_dots[i].x == tmp_dots[j].x) && (plane_dots[i].y == tmp_dots[j].y))
					{
						return 1;
					}
				}
			}
		}
	}
	
	return 0;
}


void draw_plane(uint8_t x, uint8_t y, uint8_t dir, uint8_t led_value)
{	
	uint8_t i;

	get_plane_dots(plane_dots, x, y, dir);
	
	for(i = 0; i<10; i++)
	{
		draw_dot(plane_dots[i].x, plane_dots[i].y, led_value);
	}
}

dot_t random_cursor_position()
{
	dot_t pos;
	
	pos.x = random() % 10;
	pos.y = random() % 10;
	
	while(validate_dot_position(pos.x, pos.y) == 1)
	{
		pos.x = random() % 10;
		pos.y = random() % 10;
	}
	
	return pos;
}

dot_t  random_plane_position(uint8_t *dir)
{
	uint8_t misses = 0;	
	dot_t pos = random_cursor_position();
	
	while(validate_plane_position(pos.x, pos.y, *dir, VALIDATE_COMPLETE) == 1)
	{
		misses++;
		
		if (misses == 100)
		{
			*dir += 1;
			misses = 0;
			
			if (*dir > PLANE_LEFT)
			{
				break;
			}
		}
		
		pos = random_cursor_position();
	}
	
	return pos;
}

void init_game(uint8_t *start_x, uint8_t *start_y, uint8_t *dir, uint8_t *dot_type, uint8_t *saved_dot_type)
{
	uint8_t i;
	
	LINES_DDR |= 0xFF;
	COLS0_7_DDR |= 0xFF;
	COLS8_9_DDR |= 0xC0;
	BUTTONS_DDR |= 0xC0;
	
	LINES &= 0x00;
	COLS0_7 &= 0x00;
	COLS8_9 &= 0x00;
	BUTTONS_PORT |= 0x1F;
	 
	map = (display_t*) malloc(sizeof(display_t));
	radar = (display_t*) malloc(sizeof(display_t));
	
	map->pixels = (uint8_t**) malloc(sizeof(uint8_t*) * 10);
	radar->pixels = (uint8_t**) malloc(sizeof(uint8_t*) * 10);
	
	 for (i = 0; i < 10; i++)
	 {
		map->pixels[i] = (uint8_t*) malloc (sizeof(uint8_t) * 10);
		radar->pixels[i] = (uint8_t*) malloc (sizeof(uint8_t) * 10);
	 }
	 
	current_display = radar;
	clear_display();
 	current_display = map;
 	clear_display();
	init_random();
	
	for (i = 0; i < PLANES_NUM; i++)
	{
		planes[i].status = 10;
	}
	
	game_state = SETUP_CHOICE;
	*dir = PLANE_UP;
	
	dot_t tmp = random_plane_position(dir);
	*start_x = tmp.x;
	*start_y = tmp.y;
	skip_states = 0;
	blink_count = 0;
	blink_state = 0;
	
	*dot_type = PIXEL_BLINK;
	*saved_dot_type = *dot_type;
	
	plane_index = 0;
	hit_index = 0;
	draw_plane(*start_x, *start_y, *dir, *dot_type);
}

//Communication stuff //

void init_USART()
{
	UCSRC &= ~_BV(URSEL);
	UBRRH = 0;					
	UBRRL = 103;									//9600 baud
	UCSRB |= _BV(TXEN) | _BV(RXEN);	
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);	//8bit 
}

uint8_t pack_bcd(uint8_t x, uint8_t y)
{
	uint8_t bcd;
	
	bcd = x;
	bcd = bcd << 4;
	bcd |= y & 0x0F;
	
	return bcd;
}

void unpack_bcd(uint8_t bcd, uint8_t *x, uint8_t *y)
{
	
	*x = (bcd & 0xF0) >> 4;
	*y = bcd & 0x0F;
}


void usart_send(uint8_t data)
{
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = data;
}

uint8_t usart_recv()
{
	loop_until_bit_is_set(UCSRA, RXC);
	return UDR;
}

//----------------------------------//

int main()
{
	uint8_t cursor_x, cursor_y/*, opponent_cursor_x, opponent_cursor_y*/;
	dot_t new_cursor;
	uint8_t dot_type, saved_dot_type;
	uint8_t tmp;
	uint8_t plane_dir;

	uint8_t i, j;	
	int8_t button_pushed[6] = {0, 0, 0, 0, 0, 0};

	uint32_t timeout = 0, timeout_on = 0;
	
	uint8_t dice_tie = 1;	

	uint8_t usart_buff;
	uint8_t dice;
	uint8_t my_score = 0, opp_score = 0;
	
	init_game(&cursor_x, &cursor_y, &plane_dir, &dot_type, &saved_dot_type);
	
	init_USART();


	
// 	while(1)
// 	{
// 		if ((BUTTONS & _BV(BUTTON_MIDDLE)) == 0)
// 		{
// 			PORTD ^= _BV(PD7);
// 			break;
// 		}
// 		
// 	}
// 	
// 	while (dice_tie)
// 	{
// 		loop_until_bit_is_set(UCSRA, UDRE);
// 		dice = random() % 0x0F;
// 		UDR = dice;
// 	
// 		loop_until_bit_is_set(UCSRA, RXC);
// 		usart_buff = UDR;
// 		
// 		if (dice > usart_buff)
// 		{
// 			turn = MY_TURN;
// 			dice_tie = 0;
// 			continue;
// 		}
// 		if (dice < usart_buff)
// 		{
// 			turn = OPP_TURN;
// 			dice_tie = 0;
// 			continue;
// 		}
// 		if (dice == usart_buff)
// 		{
// 			dice_tie = 1;
// 			continue;
// 		}
// 	}
	
	if (turn == MY_TURN)
		PORTD |= _BV(PD7);
	if (turn == OPP_TURN)
		PORTD &= ~_BV(PD7);
	

	while(1)
	{

	//Button press events:
		switch(~BUTTONS & 0x1F)
		{
			case _BV(BUTTON_RIGHT):
				if (!button_pushed[RIGHT_PUSHED])
				{
					button_pushed[RIGHT_PUSHED] = 1;
				}
			break;
			case _BV(BUTTON_LEFT):
				if (!button_pushed[LEFT_PUSHED])
				{
					button_pushed[LEFT_PUSHED] = 1;
				}
			break;
			case _BV(BUTTON_UP):
				if (!button_pushed[UP_PUSHED])
				{
					button_pushed[UP_PUSHED] = 1;
				}
			break;
			case _BV(BUTTON_DOWN):
				if (!button_pushed[DOWN_PUSHED])
				{
					button_pushed[DOWN_PUSHED] = 1;				
				}
			break;
			case _BV(BUTTON_MIDDLE):
				if (!button_pushed[MIDDLE_PUSHED])
				{
					button_pushed[MIDDLE_PUSHED] = 1;
				}				
			break;
		}
			
		switch(game_state)
		{
			case SETUP_CHOICE:
				for (i = 0; i < plane_index; i++)
				{
					draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_OFF);
					draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_ON);
				}
				
				dot_type = PIXEL_OFF;										
				draw_plane(cursor_x, cursor_y, plane_dir, dot_type);
				
			break;
			case SETUP_SAVE:
				if(plane_index < PLANES_NUM)
				{
					planes[plane_index].x = cursor_x;
					planes[plane_index].y = cursor_y;
					planes[plane_index].status = 0;
					planes[plane_index].dir = plane_dir;
					switch (plane_dir)
					{
						case PLANE_UP:
							planes[plane_index].head.x = cursor_x - 1;
							planes[plane_index].head.y = cursor_y;
						break;
						case PLANE_DOWN:
							planes[plane_index].head.x = cursor_x + 1;
							planes[plane_index].head.y = cursor_y;
						break;
						case PLANE_RIGHT:
							planes[plane_index].head.x = cursor_x ;
							planes[plane_index].head.y = cursor_y + 1;
						break;
						case PLANE_LEFT:
							planes[plane_index].head.x = cursor_x;
							planes[plane_index].head.y = cursor_y  - 1;
						break;
					}
					plane_index++;
					
					draw_plane(cursor_x, cursor_y, plane_dir, PIXEL_ON);
				}						
			
				if (plane_index != PLANES_NUM)
				{
					dot_t tmp = random_plane_position(&plane_dir);
					cursor_x = tmp.x;
					cursor_y = tmp.y;	
					game_state = SETUP_CHOICE;
				}
				else
				{
					game_state = READY;
					loop_until_bit_is_set(UCSRA, UDRE);
					UDR = ACK;		
				}
			break;
			case READY:
				PORTD ^= _BV(PD7);
				usart_send(ACK);
				usart_recv();
				
				game_state = IN_GAME;
				
				switch (turn)
				{
					
					case MY_TURN:
						//Generate cursor
						new_cursor = random_cursor_position();
						cursor_x = new_cursor.x;
						cursor_y = new_cursor.y;
						//send cursor + SHOT_DELAYED lable
						usart_buff = pack_bcd(cursor_x, cursor_y);
						usart_send(usart_buff);
						usart_buff = SHOT_DELAYED;
						usart_send(usart_buff);
						current_display = radar;
					break;
					case OPP_TURN:
						current_display = map;
						//Get cursor
						usart_buff = usart_recv();
						unpack_bcd(usart_buff, &cursor_x, &cursor_y);
						usart_recv();	//Unimportant lable
					break;						
				}
				

			break;
			case IN_GAME:
				switch (turn)
				{
					case MY_TURN:
						//display points to hits matrix
						for (i = 0; i < hit_index; i++)
						{
							draw_dot(hits[i].x, hits[i].y, PIXEL_OFF);
							draw_dot(hits[i].x, hits[i].y, PIXEL_ON);
						}
				
						dot_type = PIXEL_OFF;										
						draw_dot(cursor_x, cursor_y, dot_type);

					break;
					
					case OPP_TURN:
						//display points to map matrix
						for (i = 0; i < PLANES_NUM; i++)
						{
							draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_OFF);
							draw_plane(planes[i].x, planes[i].y, planes[i].dir, PIXEL_ON);
						}
				
						dot_type = PIXEL_OFF;										
						draw_dot(cursor_x, cursor_y, dot_type);							
					break;
				}
			break;
						
			case GAME_END:
				if (my_score == PLANES_NUM)
				{
					//Blink led for victory
					PORTD ^= _BV(PD7);
				}
			break;
		} 
			
			//Button readings
			switch (game_state)
			{
				case SETUP_CHOICE:
				//Button release events:
					//Left button
					if ((BUTTONS & _BV(BUTTON_LEFT)) && button_pushed[LEFT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_LEFT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_LEFT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  - 1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y--;
							}					
						}
						button_pushed[LEFT_PUSHED] = 0;				
					}
			
					//Right button		
					if ((BUTTONS & _BV(BUTTON_RIGHT)) && button_pushed[RIGHT_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_RIGHT, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_RIGHT;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x, cursor_y  +1, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_y++;
							}
						}
						button_pushed[RIGHT_PUSHED] = 0;				
					}	
			
					//Up button
					if((BUTTONS & _BV(BUTTON_UP)) && button_pushed[UP_PUSHED])
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_UP, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_UP;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;					
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x - 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x--;
							}					
						}
						button_pushed[UP_PUSHED] = 0;			
					}	
			
					//Down button
					if ((BUTTONS & _BV(BUTTON_DOWN))  && button_pushed[DOWN_PUSHED] )
					{	
						if (button_pushed[FUNCTION] == 1)
						{
							//Roteste
							if (validate_plane_position(cursor_x, cursor_y, PLANE_DOWN, VALIDATE_BOUNDARY) != 1)
							{
								plane_dir = PLANE_DOWN;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							//Deplaseaza
							if (validate_plane_position(cursor_x + 1, cursor_y, plane_dir, VALIDATE_BOUNDARY) != 1)
							{
								cursor_x++;
							}					
						}
						button_pushed[DOWN_PUSHED] = 0;
					}	
			
					//Middle button
					if ((BUTTONS & _BV(BUTTON_MIDDLE)) && button_pushed[MIDDLE_PUSHED])
					{	
						button_pushed[FUNCTION]++;
				
						if (button_pushed[FUNCTION] == 2)
						{
							if (validate_plane_position(cursor_x, cursor_y, plane_dir, VALIDATE_COMPLETE) != 1)
							{
								game_state = SETUP_SAVE;
							}
							button_pushed[FUNCTION] = 0;
							timeout_on = 0;
						}
						else
						{
							timeout = 0;
							timeout_on = 1;
						}
						button_pushed[MIDDLE_PUSHED] = 0;
					}	
				break;
				
				case IN_GAME:
					if (skip_states)
					{
						break;
					}
					switch (turn)
					{
						case MY_TURN:
							//move cursor and shoot

							//Left button
							if ((BUTTONS & _BV(BUTTON_LEFT)) && button_pushed[LEFT_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x, cursor_y  - 1)  != 1)
								{
									cursor_y--;
								}					
								button_pushed[LEFT_PUSHED] = 0;				
							}
			
							//Right button		
							if ((BUTTONS & _BV(BUTTON_RIGHT)) && button_pushed[RIGHT_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x, cursor_y  +1) != 1)
								{
									cursor_y++;
								}

								button_pushed[RIGHT_PUSHED] = 0;				
							}	
			
							//Up button
							if((BUTTONS & _BV(BUTTON_UP)) && button_pushed[UP_PUSHED])
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x - 1, cursor_y) != 1)
								{
									cursor_x--;
								}					
								button_pushed[UP_PUSHED] = 0;			
							}	
			
							//Down button
							if ((BUTTONS & _BV(BUTTON_DOWN))  && button_pushed[DOWN_PUSHED] )
							{	
								//Deplaseaza
								if (validate_dot_position(cursor_x + 1, cursor_y) != 1)
								{
									cursor_x++;
								}					
								button_pushed[DOWN_PUSHED] = 0;
							}	
			
							//Middle button
							if ((BUTTONS & _BV(BUTTON_MIDDLE)) && button_pushed[MIDDLE_PUSHED])
							{	
								button_pushed[FUNCTION]++;
				
								if (button_pushed[FUNCTION] == 2)
								{
									//OBS: il obliga sa traga intr-un loc neincercat
									if (validate_dot_position(cursor_x, cursor_y) != 1)
									{
										//Hit target
										game_state = SHOOT;
									}
									button_pushed[FUNCTION] = 0;
									timeout_on = 0;
								}
								else
								{
									timeout = 0;
									timeout_on = 1;
								}
								button_pushed[MIDDLE_PUSHED] = 0;
							}	
						break;
						
						case OPP_TURN:
							//nothing, just sit and watch
							;
						break;
					}
				break;
			}
			

			
			
			switch(game_state)
			{
				case SETUP_CHOICE:
						draw_plane(cursor_x, cursor_y, plane_dir, PIXEL_BLINK);
				break;
				
				case IN_GAME:
					if (skip_states)
					{
						skip_states = 0;
						break;
					}
						
					switch (turn)
					{
						case MY_TURN:
							//Send cursor position
							usart_buff = pack_bcd(cursor_x, cursor_y);
							usart_send(usart_buff);
							usart_send(SHOT_DELAYED);
							usart_recv();	//Sync
							draw_dot(cursor_x, cursor_y, PIXEL_BLINK);
						break;
						
						case OPP_TURN:
							//Receive package
							usart_buff = usart_recv();
							unpack_bcd(usart_buff, &cursor_x, &cursor_y);
							usart_buff = usart_recv();
							usart_send(ACK);
						
							if (usart_buff == SHOT_DECIDED)
							{
								//Shot decided
																
								usart_buff = SHOT_MISS;
								//Plane hit: kill or hit?
								for (i = 0; i < PLANES_NUM; i++)
								{
									if (planes[i].status == 0)
									{
										continue;
									}
									get_plane_dots(plane_dots, planes[i].x, planes[i].y, planes[i].dir);
									//Headshot?
									if (planes[i].head.x == cursor_x && planes[i].head.y == cursor_y)
									{
										usart_buff = SHOT_KILL;
										planes[i].status = 0;
										//Remove plane
										for (j= 0; j < 10; j++)
										{
											draw_dot(plane_dots[i].x, plane_dots[i].y, PIXEL_OFF);
										}
										break;
									}
									else
									{
									//Regular shot. Determine which plane was hit and update its status.
										usart_buff = SHOT_HIT;
										for (j = 0; j < 10; j++)
										{
											//TODO: plane_dots imi da toate punctele, si cele lovite anterior
											//TODO: Trebuie obligat oponentu sa nu traga in acelasi loc de doua ori
											if (plane_dots[j].x == cursor_x && plane_dots[j].y == cursor_y)
											{
												planes[i].status--;
												if (planes[i].status == 0)
												{
													usart_buff = SHOT_KILL;
													
												}
												//Remove piece
												draw_dot(plane_dots[i].x, plane_dots[i].y, PIXEL_OFF);
												break;
											}
										}
									}
								}
								
								if (usart_buff == SHOT_KILL)
								{
									opp_score++;
								}
								
								//Send feedback and change game_state
								usart_send(usart_buff);
								turn = MY_TURN;
								game_state = READY;
							}
							else
							{
							//Otherwise, draw cursor
								draw_dot(cursor_x, cursor_y, PIXEL_BLINK);
							}
						break;
					}
				break;
				case SHOOT:
					switch (turn)
					{
						case MY_TURN:
							//Send cursor position
							usart_buff = pack_bcd(cursor_x, cursor_y);
							usart_send(usart_buff);
							//Send SHOT_DECIDED
							usart_buff = SHOT_DECIDED;
							usart_send(usart_buff);
							//Receive feedback
							usart_buff = usart_recv();
						
							hits[hit_index].x = cursor_x;
							hits[hit_index].y = cursor_y;
							hits[hit_index].type = usart_buff;
							hit_index++;
							
							if (usart_buff == SHOT_KILL)
							{
								my_score++;
								PORTD ^= _BV(PD7);
							
							}
							
							turn = OPP_TURN;
							game_state = READY;
						break;
						case OPP_TURN:
						//Not for OPP_TURN
							;
						break;
					}
				break;
			}
			
		
			//Keyboard timeout
			if (timeout_on)
			{
				timeout++;	
				if (timeout == TIMEOUT)
				{
					button_pushed[FUNCTION] = 0;
					//PORTD ^= _BV(PD7);
				}
			}
		
			// Refresh display //
			blink_count++;
			refresh_display();
			blink_count %= BLINK_FREQ;
			if (blink_count == 0)
			{
				blink_state = (blink_state + 1) % 2;
				//PORTD ^= _BV(PD7);
			}
			//-----------------------//
		
			//Check scores//
		
			//TODO: muta asta in bucla principala
			if (my_score == PLANES_NUM || opp_score == PLANES_NUM)
			{
				game_state = GAME_END;
			}
		
		}
	
		free(map);
		free(radar);

	return 0;
}